@@ -8,22 +8,27 @@ import time
 from collections import defaultdict
 from dataclasses import dataclass, field
 from enum import Enum
-from typing import Optional
+from typing import Any, DefaultDict, Dict, Iterable, Iterator, List, Optional, Tuple, Union
 
 from dataclasses_json import CatchAll, LetterCase, Undefined
 from dataclasses_json import config as dataclass_config
 from dataclasses_json import dataclass_json
 from tqdm.auto import tqdm
 
+from ..config import SHConfig
 from ..constants import RequestType
 from ..data_collections import DataCollection
 from ..geometry import CRS, BBox, Geometry
+from ..type_utils import Json, JsonDict
 from .base import BaseCollection, SentinelHubFeatureIterator, SentinelHubService
 from .process import SentinelHubRequest
 from .utils import datetime_config, enum_config, remove_undefined
 
 LOGGER = logging.getLogger(__name__)
 
+BatchRequestType = Union[str, dict, "BatchRequest"]
+BatchCollectionType = Union[str, dict, "BatchCollection"]
+
 
 class SentinelHubBatch(SentinelHubService):
     """An interface class for Sentinel Hub Batch API
@@ -33,11 +38,19 @@ class SentinelHubBatch(SentinelHubService):
 
     # pylint: disable=too-many-public-methods
     @staticmethod
-    def _get_service_url(base_url):
+    def _get_service_url(base_url: str) -> str:
         """Provides URL to Catalog API"""
         return f"{base_url}/api/v1/batch"
 
-    def create(self, sentinelhub_request, tiling_grid, output=None, bucket_name=None, description=None, **kwargs):
+    def create(
+        self,
+        sentinelhub_request: Union[SentinelHubRequest, Dict[str, Any]],
+        tiling_grid: Dict[str, Any],
+        output: Optional[Dict[str, Any]] = None,
+        bucket_name: Optional[str] = None,
+        description: Optional[str] = None,
+        **kwargs: Any,
+    ) -> "BatchRequest":
         """Create a new batch request
 
         `Batch API reference
@@ -45,23 +58,17 @@ class SentinelHubBatch(SentinelHubService):
 
         :param sentinelhub_request: An instance of SentinelHubRequest class containing all request parameters.
             Alternatively, it can also be just a payload dictionary for Process API request
-        :type sentinelhub_request: SentinelHubRequest or dict
         :param tiling_grid: A dictionary with tiling grid parameters. It can be built with `tiling_grid` method
-        :type tiling_grid: dict
         :param output: A dictionary with output parameters. It can be built with `output` method. Alternatively, one
             can set `bucket_name` parameter instead.
-        :type output: dict or None
         :param bucket_name: A name of an S3 bucket where to save data. Alternatively, one can set `output` parameter
             to specify more output parameters.
-        :type bucket_name: str or None
         :param description: A description of a batch request
-        :type description: str or None
         :param kwargs: Any other arguments to be added to a dictionary of parameters.
         :return: An instance of `SentinelHubBatch` object that represents a newly created batch request.
-        :rtype: BatchRequest
         """
         if isinstance(sentinelhub_request, SentinelHubRequest):
-            sentinelhub_request = sentinelhub_request.download_list[0].post_values
+            sentinelhub_request = sentinelhub_request.download_list[0].post_values  # type: ignore[assignment]
 
         if not isinstance(sentinelhub_request, dict):
             raise ValueError(
@@ -82,22 +89,20 @@ class SentinelHubBatch(SentinelHubService):
         url = self._get_process_url()
         request_info = self.client.get_json(url, post_values=payload, use_session=True)
 
-        return BatchRequest.from_dict(request_info)
+        return BatchRequest.from_dict(request_info)  # type: ignore[attr-defined]
 
     @staticmethod
-    def tiling_grid(grid_id, resolution, buffer=None, **kwargs):
+    def tiling_grid(
+        grid_id: int, resolution: float, buffer: Optional[Tuple[int, int]] = None, **kwargs: Any
+    ) -> JsonDict:
         """A helper method to build a dictionary with tiling grid parameters
 
         :param grid_id: An ID of a tiling grid
-        :type grid_id: int
         :param resolution: A grid resolution
-        :type resolution: float or int
         :param buffer: Optionally, a buffer around each tile can be defined. It can be defined with a tuple of integers
             `(buffer_x, buffer_y)`, which specifies a number of buffer pixels in horizontal and vertical directions.
-        :type buffer: (int, int) or None
         :param kwargs: Any other arguments to be added to a dictionary of parameters
         :return: A dictionary with parameters
-        :rtype: dict
         """
         payload = {"id": grid_id, "resolution": resolution, **kwargs}
         if buffer:
@@ -107,39 +112,30 @@ class SentinelHubBatch(SentinelHubService):
     @staticmethod
     def output(
         *,
-        default_tile_path=None,
-        overwrite=None,
-        skip_existing=None,
-        cog_output=None,
-        cog_parameters=None,
-        create_collection=None,
-        collection_id=None,
-        responses=None,
-        **kwargs,
-    ):
+        default_tile_path: Optional[str] = None,
+        overwrite: Optional[bool] = None,
+        skip_existing: Optional[bool] = None,
+        cog_output: Optional[bool] = None,
+        cog_parameters: Optional[Dict[str, Any]] = None,
+        create_collection: Optional[bool] = None,
+        collection_id: Optional[str] = None,
+        responses: Optional[List[str]] = None,
+        **kwargs: Any,
+    ) -> Dict[str, Any]:
         """A helper method to build a dictionary with tiling grid parameters
 
         :param default_tile_path: A path or a template on an s3 bucket where to store results. More info at Batch API
             documentation
-        :type default_tile_path: str or None
         :param overwrite: A flag specifying if a request should overwrite existing outputs without failing
-        :type overwrite: bool or None
         :param skip_existing: A flag specifying if existing outputs should be overwritten
-        :type skip_existing: bool or None
         :param cog_output: A flag specifying if outputs should be written in COGs (cloud-optimized GeoTIFFs )or
             normal GeoTIFFs
-        :type cog_output: bool or None
         :param cog_parameters: A dictionary specifying COG creation parameters
-        :type cog_parameters: dict or None
         :param create_collection: If True the results will be written in COGs and a batch collection will be created
-        :type create_collection: bool or None
         :param collection_id: If True results will be added to an existing collection
-        :type collection_id: str or None
         :param responses: Specification of path template for individual outputs/responses
-        :type responses: list or None
         :param kwargs: Any other arguments to be added to a dictionary of parameters
         :return: A dictionary of output parameters
-        :rtype: dict
         """
         return remove_undefined(
             {
@@ -155,7 +151,7 @@ class SentinelHubBatch(SentinelHubService):
             }
         )
 
-    def iter_tiling_grids(self, **kwargs):
+    def iter_tiling_grids(self, **kwargs: Any) -> SentinelHubFeatureIterator:
         """An iterator over tiling grids
 
         `Batch API reference
@@ -163,7 +159,6 @@ class SentinelHubBatch(SentinelHubService):
 
         :param kwargs: Any other request query parameters
         :return: An iterator over tiling grid definitions
-        :rtype: Iterator[dict]
         """
         return SentinelHubFeatureIterator(
             client=self.client,
@@ -172,47 +167,43 @@ class SentinelHubBatch(SentinelHubService):
             exception_message="Failed to obtain information about available tiling grids",
         )
 
-    def get_tiling_grid(self, grid_id):
+    def get_tiling_grid(self, grid_id: Union[int, str]) -> JsonDict:
         """Provides a single tiling grid
 
         `Batch API reference
         <https://docs.sentinel-hub.com/api/latest/reference/#operation/getBatchTilingGridProperties>`__
 
         :param grid_id: An ID of a requested tiling grid
-        :type grid_id: str or int
         :return: A tiling grid definition
-        :rtype: dict
         """
-        return self.client.get_json(self._get_tiling_grids_url(grid_id), use_session=True)
+        url = self._get_tiling_grids_url(grid_id)
+        return self.client.get_json_dict(url=url, use_session=True)
 
-    def iter_requests(self, user_id=None, search=None, sort=None, **kwargs):
+    def iter_requests(
+        self, user_id: Optional[str] = None, search: Optional[str] = None, sort: Optional[str] = None, **kwargs: Any
+    ) -> Iterator["BatchRequest"]:
         """Iterate existing batch requests
 
         `Batch API reference
         <https://docs.sentinel-hub.com/api/latest/reference/#operation/getAllBatchProcessRequests>`__
 
         :param user_id: Filter requests by a user id who defined a request
-        :type user_id: str or None
         :param search: A search query to filter requests
-        :type search: str or None
         :param sort: A sort query
-        :type sort: str or None
         :param kwargs: Any additional parameters to include in a request query
         :return: An iterator over existing batch requests
-        :rtype: Iterator[BatchRequest]
         """
         params = remove_undefined({"userid": user_id, "search": search, "sort": sort, **kwargs})
         feature_iterator = SentinelHubFeatureIterator(
             client=self.client, url=self._get_process_url(), params=params, exception_message="No requests found"
         )
         for request_info in feature_iterator:
-            yield BatchRequest.from_dict(request_info)
+            yield BatchRequest.from_dict(request_info)  # type: ignore[attr-defined]
 
-    def get_latest_request(self):
+    def get_latest_request(self) -> "BatchRequest":
         """Provides a batch request that has been created the latest
 
         :return: Batch request info
-        :rtype: BatchRequest
         """
         latest_request_iter = self.iter_requests(sort="created:desc", count=1)
         try:
@@ -220,20 +211,25 @@ class SentinelHubBatch(SentinelHubService):
         except StopIteration as exception:
             raise ValueError("No batch request is available") from exception
 
-    def get_request(self, batch_request):
+    def get_request(self, batch_request: BatchRequestType) -> "BatchRequest":
         """Collects information about a single batch request
 
         `Batch API reference
         <https://docs.sentinel-hub.com/api/latest/reference/#operation/getSingleBatchProcessRequestById>`__
 
         :return: Batch request info
-        :rtype: BatchRequest
         """
         request_id = self._parse_request_id(batch_request)
         request_info = self.client.get_json(url=self._get_process_url(request_id), use_session=True)
-        return BatchRequest.from_dict(request_info)
+        return BatchRequest.from_dict(request_info)  # type: ignore[attr-defined]
 
-    def update_request(self, batch_request, output=None, description=None, **kwargs):
+    def update_request(
+        self,
+        batch_request: BatchRequestType,
+        output: Optional[Dict[str, Any]] = None,
+        description: Optional[str] = None,
+        **kwargs: Any,
+    ) -> Json:
         """Update batch job request parameters
 
         `Batch API reference
@@ -244,11 +240,8 @@ class SentinelHubBatch(SentinelHubService):
 
         :param batch_request: It could be a batch request object, a raw batch request payload or only a batch
             request ID.
-        :type batch_request: BatchRequest or dict or str
         :param output: A dictionary with output parameters to be updated.
-        :type output: dict or None
         :param description: A description of a batch request to be updated.
-        :type description: str or None
         :param kwargs: Any other arguments to be added to a dictionary of parameters.
         """
         request_id = self._parse_request_id(batch_request)
@@ -257,7 +250,7 @@ class SentinelHubBatch(SentinelHubService):
             url=self._get_process_url(request_id), post_values=payload, request_type=RequestType.PUT, use_session=True
         )
 
-    def delete_request(self, batch_request):
+    def delete_request(self, batch_request: BatchRequestType) -> Json:
         """Delete a batch job request
 
         `Batch API reference
@@ -265,36 +258,33 @@ class SentinelHubBatch(SentinelHubService):
 
         :param batch_request: It could be a batch request object, a raw batch request payload or only a batch
             request ID.
-        :type batch_request: BatchRequest or dict or str
         """
         request_id = self._parse_request_id(batch_request)
         return self.client.get_json(
             url=self._get_process_url(request_id), request_type=RequestType.DELETE, use_session=True
         )
 
-    def start_analysis(self, batch_request):
+    def start_analysis(self, batch_request: BatchRequestType) -> Json:
         """Starts analysis of a batch job request
 
         `Batch API reference <https://docs.sentinel-hub.com/api/latest/reference/#operation/batchAnalyse>`__
 
         :param batch_request: It could be a batch request object, a raw batch request payload or only a batch
             request ID.
-        :type batch_request: BatchRequest or dict or str
         """
         return self._call_job(batch_request, "analyse")
 
-    def start_job(self, batch_request):
+    def start_job(self, batch_request: BatchRequestType) -> Json:
         """Starts running a batch job
 
         `Batch API reference <https://docs.sentinel-hub.com/api/latest/reference/#operation/batchStartProcessRequest>`__
 
         :param batch_request: It could be a batch request object, a raw batch request payload or only a batch
             request ID.
-        :type batch_request: BatchRequest or dict or str
         """
         return self._call_job(batch_request, "start")
 
-    def cancel_job(self, batch_request):
+    def cancel_job(self, batch_request: BatchRequestType) -> Json:
         """Cancels a batch job
 
         `Batch API reference
@@ -302,11 +292,10 @@ class SentinelHubBatch(SentinelHubService):
 
         :param batch_request: It could be a batch request object, a raw batch request payload or only a batch
             request ID.
-        :type batch_request: BatchRequest or dict or str
         """
         return self._call_job(batch_request, "cancel")
 
-    def restart_job(self, batch_request):
+    def restart_job(self, batch_request: BatchRequestType) -> Json:
         """Restarts only those parts of a job that failed
 
         `Batch API reference
@@ -314,23 +303,21 @@ class SentinelHubBatch(SentinelHubService):
 
         :param batch_request: It could be a batch request object, a raw batch request payload or only a batch
             request ID.
-        :type batch_request: BatchRequest or dict or str
         """
         return self._call_job(batch_request, "restartpartial")
 
-    def iter_tiles(self, batch_request, status=None, **kwargs):
+    def iter_tiles(
+        self, batch_request: BatchRequestType, status: Union[None, "BatchTileStatus", str] = None, **kwargs: Any
+    ) -> SentinelHubFeatureIterator:
         """Iterate over info about batch request tiles
 
         `Batch API reference <https://docs.sentinel-hub.com/api/latest/reference/#operation/getAllBatchProcessTiles>`__
 
         :param batch_request: It could be a batch request object, a raw batch request payload or only a batch
             request ID.
-        :type batch_request: BatchRequest or dict or str
         :param status: A filter to obtain only tiles with a certain status
-        :type status: BatchTileStatus or str or None
         :param kwargs: Any additional parameters to include in a request query
         :return: An iterator over information about each tile
-        :rtype: Iterator[dict]
         """
         request_id = self._parse_request_id(batch_request)
         if isinstance(status, BatchTileStatus):
@@ -343,46 +330,39 @@ class SentinelHubBatch(SentinelHubService):
             exception_message="No tiles found, please run analysis on batch request before calling this method",
         )
 
-    def get_tile(self, batch_request, tile_id):
+    def get_tile(self, batch_request: BatchRequestType, tile_id: Optional[int]) -> JsonDict:
         """Provides information about a single batch request tile
 
         `Batch API reference <https://docs.sentinel-hub.com/api/latest/reference/#operation/getBatchTileById>`__
 
         :param batch_request: It could be a batch request object, a raw batch request payload or only a batch
             request ID.
-        :type batch_request: BatchRequest or dict or str
         :param tile_id: An ID of a tile
-        :type tile_id: int or None
         :return: Information about a tile
-        :rtype: dict
         """
         request_id = self._parse_request_id(batch_request)
         url = self._get_tiles_url(request_id, tile_id=tile_id)
-        return self.client.get_json(url, use_session=True)
+        return self.client.get_json_dict(url, use_session=True)
 
-    def reprocess_tile(self, batch_request, tile_id):
+    def reprocess_tile(self, batch_request: BatchRequestType, tile_id: Optional[int]) -> Json:
         """Reprocess a single failed tile
 
         `Batch API reference <https://docs.sentinel-hub.com/api/latest/reference/#operation/restartBatchTileById>`__
 
         :param batch_request: It could be a batch request object, a raw batch request payload or only a batch
             request ID.
-        :type batch_request: BatchRequest or dict or str
         :param tile_id: An ID of a tile
-        :type tile_id: int or None
         """
-        self._call_job(batch_request, f"tiles/{tile_id}/restart")
+        return self._call_job(batch_request, f"tiles/{tile_id}/restart")
 
-    def iter_collections(self, search=None, **kwargs):
+    def iter_collections(self, search: Optional[str] = None, **kwargs: Any) -> SentinelHubFeatureIterator:
         """Iterate over batch collections
 
         `Batch API reference <https://docs.sentinel-hub.com/api/latest/reference/#operation/getAllBatchCollections>`__
 
         :param search: A search query to filter collections
-        :type search: str or None
         :param kwargs: Any additional parameters to include in a request query
         :return: An iterator over existing batch collections
-        :rtype: Iterator[dict]
         """
         return SentinelHubFeatureIterator(
             client=self.client,
@@ -391,41 +371,36 @@ class SentinelHubBatch(SentinelHubService):
             exception_message="Failed to obtain information about available Batch collections",
         )
 
-    def get_collection(self, collection_id):
+    def get_collection(self, collection_id: str) -> JsonDict:
         """Get batch collection by its id
 
         `Batch API reference
         <https://docs.sentinel-hub.com/api/latest/reference/#operation/getSingleBatchCollectionById>`__
 
         :param collection_id: A batch collection id
-        :type collection_id: str
         :return: A dictionary of the collection parameters
-        :rtype: dict
         """
-        return self.client.get_json(url=self._get_collections_url(collection_id), use_session=True)["data"]
+        url = self._get_collections_url(collection_id)
+        return self.client.get_json_dict(url=url, use_session=True, extract_key="data")
 
-    def create_collection(self, collection):
+    def create_collection(self, collection: Union["BatchCollection", dict]) -> JsonDict:
         """Create a new batch collection
 
         `Batch API reference <https://docs.sentinel-hub.com/api/latest/reference/#operation/createNewBatchCollection>`__
 
         :param collection: Batch collection definition
-        :type collection: BatchCollection or dict
         :return: A dictionary of a newly created collection
-        :rtype: dict
         """
         collection_payload = self._parse_collection_to_dict(collection)
-        return self.client.get_json(url=self._get_collections_url(), post_values=collection_payload, use_session=True)[
-            "data"
-        ]
+        url = self._get_collections_url()
+        return self.client.get_json_dict(url=url, post_values=collection_payload, use_session=True, extract_key="data")
 
-    def update_collection(self, collection):
+    def update_collection(self, collection: Union["BatchCollection", dict]) -> Json:
         """Update an existing batch collection
 
         `Batch API reference <https://docs.sentinel-hub.com/api/latest/reference/#operation/updateBatchCollection>`__
 
         :param collection: Batch collection definition
-        :type collection: BatchCollection or dict
         """
         collection_id = self._parse_collection_id(collection)
         return self.client.get_json(
@@ -435,47 +410,46 @@ class SentinelHubBatch(SentinelHubService):
             use_session=True,
         )
 
-    def delete_collection(self, collection):
+    def delete_collection(self, collection: BatchCollectionType) -> Json:
         """Delete an existing batch collection
 
         `Batch API reference <https://docs.sentinel-hub.com/api/latest/reference/#operation/deleteBatchCollection>`__
 
         :param collection: Batch collection id or object
-        :type collection: str or BatchCollection
         """
         collection_id = self._parse_collection_id(collection)
         return self.client.get_json(
             url=self._get_collections_url(collection_id), request_type=RequestType.DELETE, use_session=True
         )
 
-    def _call_job(self, batch_request, endpoint_name):
+    def _call_job(self, batch_request: BatchRequestType, endpoint_name: str) -> Json:
         """Makes a POST request to the service that triggers a processing job"""
         request_id = self._parse_request_id(batch_request)
         job_url = f"{self._get_process_url(request_id)}/{endpoint_name}"
         return self.client.get_json(url=job_url, request_type=RequestType.POST, use_session=True)
 
-    def _get_process_url(self, request_id=None):
+    def _get_process_url(self, request_id: Optional[str] = None) -> str:
         """Creates a URL for process endpoint"""
         url = f"{self.service_url}/process"
         if request_id:
             return f"{url}/{request_id}"
         return url
 
-    def _get_tiles_url(self, request_id, tile_id=None):
+    def _get_tiles_url(self, request_id: str, tile_id: Union[None, str, int] = None) -> str:
         """Creates a URL for tiles endpoint"""
         url = f"{self._get_process_url(request_id)}/tiles"
         if tile_id:
             return f"{url}/{tile_id}"
         return url
 
-    def _get_tiling_grids_url(self, grid_id=None):
+    def _get_tiling_grids_url(self, grid_id: Union[None, str, int] = None) -> str:
         """Creates a URL for tiling grids endpoint"""
         url = f"{self.service_url}/tilinggrids"
         if grid_id:
             return f"{url}/{grid_id}"
         return url
 
-    def _get_collections_url(self, collection_id=None):
+    def _get_collections_url(self, collection_id: Optional[str] = None) -> str:
         """Creates a URL for batch collections endpoint"""
         url = f"{self.service_url}/collections"
         if collection_id:
@@ -483,7 +457,7 @@ class SentinelHubBatch(SentinelHubService):
         return url
 
     @staticmethod
-    def _parse_request_id(data):
+    def _parse_request_id(data: BatchRequestType) -> str:
         """Parses batch request id from multiple possible inputs"""
         if isinstance(data, BatchRequest):
             return data.request_id
@@ -494,7 +468,7 @@ class SentinelHubBatch(SentinelHubService):
         raise ValueError(f"Expected a BatchRequest, dictionary or a string, got {data}.")
 
     @staticmethod
-    def _parse_collection_id(data):
+    def _parse_collection_id(data: BatchCollectionType) -> Optional[str]:
         """Parses batch collection id from multiple possible inputs"""
         if isinstance(data, (BatchCollection, DataCollection)):
             return data.collection_id
@@ -505,10 +479,10 @@ class SentinelHubBatch(SentinelHubService):
         raise ValueError(f"Expected a BatchCollection dataclass, dictionary or a string, got {data}.")
 
     @staticmethod
-    def _parse_collection_to_dict(data):
+    def _parse_collection_to_dict(data: Union["BatchCollection", dict]) -> dict:
         """Constructs a dictionary from given object"""
         if isinstance(data, BatchCollection):
-            return data.to_dict()
+            return data.to_dict()  # type: ignore[attr-defined]
         if isinstance(data, dict):
             return data
         raise ValueError(f"Expected either a BatchCollection or a dict, got {data}.")
@@ -581,27 +555,25 @@ class BatchRequest:
         "tile_count",
     ]
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """A representation that shows the basic parameters of a batch job"""
         repr_params = {name: getattr(self, name) for name in self._REPR_PARAM_NAMES if getattr(self, name) is not None}
         repr_params_str = "\n  ".join(f"{name}={value}" for name, value in repr_params.items())
         return f"{self.__class__.__name__}(\n  {repr_params_str}\n  ...\n)"
 
     @property
-    def evalscript(self):
+    def evalscript(self) -> str:
         """Provides an evalscript used by a batch request
 
         :return: An evalscript
-        :rtype: str
         """
         return self.process_request["evalscript"]
 
     @property
-    def bbox(self):
+    def bbox(self) -> BBox:
         """Provides a bounding box used by a batch request
 
         :return: An area bounding box together with CRS
-        :rtype: BBox
         :raises: ValueError
         """
         bbox, _, crs = self._parse_bounds_payload()
@@ -610,11 +582,10 @@ class BatchRequest:
         return BBox(bbox, crs)
 
     @property
-    def geometry(self):
+    def geometry(self) -> Geometry:
         """Provides a geometry used by a batch request
 
         :return: An area geometry together with CRS
-        :rtype: Geometry
         :raises: ValueError
         """
         _, geometry, crs = self._parse_bounds_payload()
@@ -622,11 +593,13 @@ class BatchRequest:
             raise ValueError("Geometry is not defined for this batch request")
         return Geometry(geometry, crs)
 
-    def raise_for_status(self, status=BatchRequestStatus.FAILED):
+    def raise_for_status(
+        self,
+        status: Union[str, BatchRequestStatus, Iterable[Union[str, BatchRequestStatus]]] = BatchRequestStatus.FAILED,
+    ) -> None:
         """Raises an error in case batch request has a given status
 
         :param status: One or more status codes on which to raise an error. The default is `'FAILED'`.
-        :type status: str or list(str) or BatchRequestStatus or list(BatchRequestStatus)
         :raises: RuntimeError
         """
         if isinstance(status, (str, BatchRequestStatus)):
@@ -639,7 +612,7 @@ class BatchRequest:
                 f"Raised for batch request {self.request_id} with status {self.status.value}{formatted_error_message}"
             )
 
-    def _parse_bounds_payload(self):
+    def _parse_bounds_payload(self) -> Tuple[Optional[List[float]], Optional[list], CRS]:
         """Parses bbox, geometry and crs from batch request payload. If bbox or geometry don't exist it returns None
         instead.
         """
@@ -664,7 +637,7 @@ class BatchCollectionAdditionalData:
     """Dataclass to hold batch collection additionalData part of the payload"""
 
     other_data: CatchAll
-    bands: Optional[dict] = None
+    bands: Optional[Dict[str, Any]] = None
 
 
 class BatchCollection(BaseCollection):
@@ -681,8 +654,11 @@ _DEFAULT_ANALYSIS_SLEEP_TIME = 10
 
 
 def monitor_batch_job(
-    batch_request, config, sleep_time=_DEFAULT_SLEEP_TIME, analysis_sleep_time=_DEFAULT_ANALYSIS_SLEEP_TIME
-):
+    batch_request: BatchRequestType,
+    config: Optional[SHConfig],
+    sleep_time: int = _DEFAULT_SLEEP_TIME,
+    analysis_sleep_time: int = _DEFAULT_ANALYSIS_SLEEP_TIME,
+) -> DefaultDict[BatchTileStatus, List[dict]]:
     """A utility function that keeps checking for number of processed tiles until the given batch request finishes.
     During the process it shows a progress bar and at the end it reports information about finished and failed tiles.
 
@@ -697,17 +673,12 @@ def monitor_batch_job(
 
     :param batch_request: An object with information about a batch request. Alternatively, it could only be a batch
         request id or a payload.
-    :type batch_request: BatchRequest or dict or str.
     :param config: A configuration object with required parameters `sh_client_id`, `sh_client_secret`, and
         `sh_auth_base_url` which is used for authentication and `sh_base_url` which defines the service deployment
         where Batch API will be called.
-    :type config: SHConfig or None
     :param sleep_time: Number of seconds to sleep between consecutive progress bar updates.
-    :type sleep_time: int
     :param analysis_sleep_time: Number of seconds between consecutive status updates during analysis phase.
-    :type analysis_sleep_time: int
     :return: A dictionary mapping a tile status to a list of tile payloads.
-    :rtype: defaultdict(BatchTileStatus, list(dict))
     """
     if sleep_time < _MIN_SLEEP_TIME:
         raise ValueError(f"To avoid making too many service requests please set sleep_time>={_MIN_SLEEP_TIME}")
@@ -747,21 +718,19 @@ def monitor_batch_job(
     return tiles_per_status
 
 
-def monitor_batch_analysis(batch_request, config, sleep_time=_DEFAULT_ANALYSIS_SLEEP_TIME):
+def monitor_batch_analysis(
+    batch_request: BatchRequestType, config: Optional[SHConfig], sleep_time: int = _DEFAULT_ANALYSIS_SLEEP_TIME
+) -> BatchRequest:
     """A utility function that is waiting until analysis phase of a batch job finishes and regularly checks its status.
     In case analysis phase failed it raises an error at the end.
 
     :param batch_request: An object with information about a batch request. Alternatively, it could only be a batch
         request id or a payload.
-    :type batch_request: BatchRequest or dict or str
     :param config: A configuration object with required parameters `sh_client_id`, `sh_client_secret`, and
         `sh_auth_base_url` which is used for authentication and `sh_base_url` which defines the service deployment
         where Batch API will be called.
-    :type config: SHConfig
     :param sleep_time: Number of seconds between consecutive status updates during analysis phase.
-    :type sleep_time: int
     :return: Batch request info
-    :rtype: BatchRequest
     """
     if sleep_time < _MIN_ANALYSIS_SLEEP_TIME:
         raise ValueError(
@@ -780,12 +749,13 @@ def monitor_batch_analysis(batch_request, config, sleep_time=_DEFAULT_ANALYSIS_S
     return batch_request
 
 
-def _get_batch_tiles_per_status(batch_request, batch_client):
+def _get_batch_tiles_per_status(
+    batch_request: BatchRequestType, batch_client: SentinelHubBatch
+) -> DefaultDict[BatchTileStatus, List[dict]]:
     """A helper function that queries information about batch tiles and returns information about tiles, grouped by
     tile status.
 
     :return: A dictionary mapping a tile status to a list of tile payloads.
-    :rtype: defaultdict(BatchTileStatus, list(dict))
     """
     tiles_per_status = defaultdict(list)
 
